import java.io.IOException;
import java.io.InputStream;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.glu.GLU;
import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;






public class TerrainColor extends JFrame{

	GLCanvas canvas;
	SceneView sv;

	ArrayList<FloatBuffer> faceNormals = new ArrayList<FloatBuffer>();
	float[][] vNormals = new float[10000][3];

	public TerrainColor() {

		sv = new SceneView(); //GLEventListener-Objekt

		GLProfile glp = GLProfile.getDefault();
		GLCapabilities caps = new GLCapabilities(glp);
		canvas = new GLCanvas(caps);
		canvas.addGLEventListener(sv);

		add(canvas);


		setTitle("Terrain");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(1200,600);
		setVisible(true);

	}



	class SceneView implements GLEventListener {

		float[][] terrain = new float[100][100];
		//texturvariable
		private Texture ground;

		//private float rotation = 0.0f;

		public SceneView(){

			generateHeightField();
			calculateFaceNormals();
			calculateVertexNormals();

		}


		public void init(GLAutoDrawable drawable) {
			GL2 gl = drawable.getGL().getGL2();
			gl.glClearColor(0.8f, 0.9f, 0.9f, 0f); // grey background
			gl.glClearDepth(1.0);//Depth Buffer Setup
			gl.glEnable(GL.GL_DEPTH_TEST);//Enables Depth Testing
			//gl.glPolygonMode(GL2.GL_FRONT_AND_BACK, GL2.GL_LINE );
			float light_ambient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
			float light_diffuse[] = { 1.0f, 1.0f, 0.8f, 1.0f };
			float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float light_position[] = { 40.0f, 55.0f, 0.0f, 0.0f };
			float light_direction[] = { -40.0f, -55.0f, 0.0f, 0.0f };

			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_AMBIENT, FloatBuffer.wrap(light_ambient));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_DIFFUSE, FloatBuffer.wrap(light_diffuse));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_SPECULAR,FloatBuffer.wrap(light_specular));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_POSITION, FloatBuffer.wrap(light_position));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_SPOT_DIRECTION, FloatBuffer.wrap(light_direction));
			gl.glLightf(GL2.GL_LIGHT0, GL2.GL_SPOT_CUTOFF, 45.0f);

			gl.glEnable(GL2.GL_LIGHTING);
			gl.glEnable(GL2.GL_LIGHT0);
			gl.glEnable(GL.GL_DEPTH_TEST);
			gl.glShadeModel(GL2.GL_SMOOTH);
			gl.glClearDepth(1.0f);
			gl.glDepthFunc(GL.GL_LEQUAL);
			gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);

			try{
				InputStream stream = getClass().getResourceAsStream("grass.png");
				TextureData data =
						TextureIO.newTextureData(gl.getGLProfile(),stream, false,"png");
				ground = TextureIO.newTexture(data);
			}catch(IOException e){
				e.printStackTrace();
			}

		}

		public void display(GLAutoDrawable drawable) {

			GL2 gl = drawable.getGL().getGL2();
			GLU glu = new GLU();
			gl.glClear(GL2.GL_COLOR_BUFFER_BIT|GL.GL_DEPTH_BUFFER_BIT); //clear background & depth buffer

			gl.glMatrixMode(GL2.GL_MODELVIEW);
			gl.glLoadIdentity();
			glu.gluLookAt(35, 20, 38, 0, 0, 0, 0, 1, 0);


			// Apply texture.
			ground.enable(gl);
			ground.bind(gl);


			float[] normal = new float [3];

			float no_mat[] = { 0.0f, 0.0f, 0.0f, 1.0f };
			float mat_amb[] = { 0.4f, 0.4f, 0.4f, 1.0f };
			float mat_diffuse[] = { 0.1f, 0.8f, 0.2f, 1.0f };
			float no_shininess[] = { 0.0f };

			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_AMBIENT, FloatBuffer.wrap(mat_amb));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_DIFFUSE, FloatBuffer.wrap(mat_diffuse));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SPECULAR, FloatBuffer.wrap(no_mat));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SHININESS,FloatBuffer.wrap(no_shininess));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_EMISSION, FloatBuffer.wrap(no_mat));


			//translate whole landscape
			gl.glTranslated(-50, 0, -50);

			//The extended definition of the triangle strips
			for(int i = 0; i<terrain.length-1;i++){
				gl.glBegin(GL2.GL_TRIANGLE_STRIP);

				for(int j = 0; j<terrain[0].length-1;j++){

					//set pre-calculated normals
					normal[0] = vNormals[i+j*100][0];
					normal[1] = vNormals[i+j*100][1];
					normal[2] = vNormals[i+j*100][2];
					gl.glNormal3fv(FloatBuffer.wrap(normal));
					//Texture-coordinate
					gl.glTexCoord2f(0f,0f);
					//vertex
					gl.glVertex3d(i,terrain[i][j], j);



					normal[0] = vNormals[i+1+j*100][0];
					normal[1] = vNormals[i+1+j*100][1];
					normal[2] = vNormals[i+1+j*100][2];
					gl.glNormal3fv(FloatBuffer.wrap(normal));
					gl.glTexCoord2f(1f,0f);
					gl.glVertex3d(i+1,terrain[i+1][j], j);



					normal[0] = vNormals[i+(j+1)*100][0];
					normal[1] = vNormals[i+(j+1)*100][1];
					normal[2] = vNormals[i+(j+1)*100][2];
					gl.glNormal3fv(FloatBuffer.wrap(normal));
					gl.glTexCoord2f(1f,1f);
					gl.glVertex3d(i,terrain[i][j+1], j+1);


					normal[0] = vNormals[i+1+(j+1)*100][0];
					normal[1] = vNormals[i+1+(j+1)*100][1];
					normal[2] = vNormals[i+1+(j+1)*100][2];
					gl.glNormal3fv(FloatBuffer.wrap(normal));
					gl.glTexCoord2f(0f,1f);
					gl.glVertex3d(i+1,terrain[i+1][j+1], j+1);

				}

				gl.glEnd();
			}




		}

		public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
			GL2 gl = drawable.getGL().getGL2();
			GLU glu = new GLU();

			gl.glMatrixMode(GL2.GL_PROJECTION);
			gl.glLoadIdentity();
			glu.gluPerspective(60, w/h, 0.1, 1000);
		}

		public void dispose(GLAutoDrawable drawable) {
		}

		private void generateHeightField(){

			int[] pixels =
					ImageLoader.loadImage("http://www.medien.ifi.lmu.de/lehre/ss12/cg1/uebung/heightmap.png");

			for(int i = 0; i<terrain.length;i++){
				for(int j = 0; j<terrain[0].length;j++){
					terrain[i][j] = (float)pixels[i+j*100]/5.0f-5;
				}
			}

		}

        //new method for calculating the 
		private void calculateFaceNormals(){

			faceNormals.clear();

			float[] a = new float[3];
			float[] b = new float[3];
			float[] c = new float[3];
			float[] d = new float[3];

			float[] ip1 = new float[3];
			float[] ip2 = new float[3];
			float[] ip3 = new float[3];

			for(int i = 0; i<terrain.length-1;i++){ //99 TriangleStrips

				//For each two triangles defined by 4 Vertices, calculate the face normal
				for(int j = 0; j<terrain[0].length-1;j++){

					//Point a
					a[0] = i;
					a[1] = terrain[i][j];
					a[2] = j;
					//Point b
					b[0] = i+1;
					b[1] = terrain[i+1][j];
					b[2] = j;
					//Point c
					c[0] = i+1;
					c[1] = terrain[i+1][j+1];
					c[2] = j+1;
					//Point d
					d[0] = i;
					d[1] = terrain[i][j+1];
					d[2] = j+1;
					//vector 1
					ip1[0] = b[0] - a[0];
					ip1[1] = b[1] - a[1];
					ip1[2] = b[2] - a[2];
					//vector 2
					ip2[0] = c[0] - a[0];
					ip2[1] = c[1] - a[1];
					ip2[2] = c[2] - a[2];
					//vector 3
					ip3[0] = d[0] - c[0];
					ip3[1] = d[1] - c[1];
					ip3[2] = d[2] - c[2];

					//two normals - normals are calculated with cross product
					faceNormals.add(crossProduct(ip2,ip1));
					faceNormals.add(crossProduct(ip3,ip2));
				}
			}
		}
		
		/*calculate vertex normals: See tutorial slides for an overview. Basically,
		 * a vertex normal is calculated by averaging all adjacent face normals.
		 * The calculation is simple but long, because there are several special
		 * cases (corner and border vertices)*/
		private void calculateVertexNormals(){

			FloatBuffer fn1 = null;
			FloatBuffer fn2 = null;
			FloatBuffer fn3 = null;
			FloatBuffer fn4 = null;
			FloatBuffer fn5 = null;
			FloatBuffer fn6 = null;

			float x = 0f;
			float y = 0f;
			float z = 0f;

			float[] vertexNormal = new float[3];

			//loop through whole terrain
			for(int i = 0; i < terrain.length; i++){
				for(int j = 0; j < terrain[0].length; j++){

					//left border without corner vertices
					if(i==0 && j>0 && j<terrain[0].length-1){
						fn1 = faceNormals.get(i*2+j*198);
						fn2 = faceNormals.get(i*2+j*198+1);
						fn3 = faceNormals.get(i*2+(j-1)*198);

						x = (fn1.get(0)+fn2.get(0)+fn3.get(0))/3;
						y = (fn1.get(1)+fn2.get(1)+fn3.get(1))/3;
						z = (fn1.get(2)+fn2.get(2)+fn3.get(2))/3;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//right border without corner vertices
					if(i==terrain.length-1 && j>0 && j<terrain[0].length-1){
						fn1 = faceNormals.get((i-1)*2+(j-1)*198);
						fn2 = faceNormals.get((i-1)*2+(j-1)*198+1);
						fn3 = faceNormals.get((i-1)*2+j*198+1);

						x = (fn1.get(0)+fn2.get(0)+fn3.get(0))/3;
						y = (fn1.get(1)+fn2.get(1)+fn3.get(1))/3;
						z = (fn1.get(2)+fn2.get(2)+fn3.get(2))/3;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//lower left corner
					if(i==0 && j==0){
						fn1 = faceNormals.get(i*2+j*198);
						fn2 = faceNormals.get(i*2+j*198+1);


						x = (fn1.get(0)+fn2.get(0))/2;
						y = (fn1.get(1)+fn2.get(1))/2;
						z = (fn1.get(2)+fn2.get(2))/2;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//lower right corner
					if(i==terrain.length-1 && j==0){
						fn1 = faceNormals.get((i-1)*2+j*198+1);

						vertexNormal[0] = fn1.get(0);
						vertexNormal[1] = fn1.get(1);
						vertexNormal[2] = fn1.get(2);

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//upper right corner
					if(i==terrain.length-1 && j==terrain[0].length-1){
						fn1 = faceNormals.get((i-1)*2+(j-1)*198);
						fn2 = faceNormals.get((i-1)*2+(j-1)*198+1);

						x = (fn1.get(0)+fn2.get(0))/2;
						y = (fn1.get(1)+fn2.get(1))/2;
						z = (fn1.get(2)+fn2.get(2))/2;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//upper left corner
					if(i==0 && j==terrain[0].length-1){
						fn1 = faceNormals.get((i*2+(j-1)*198));

						vertexNormal[0] = fn1.get(0);
						vertexNormal[1] = fn1.get(1);
						vertexNormal[2] = fn1.get(2);

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//lower border without corner vertices
					if(j==0 && i>0 && i<terrain.length-1){
						fn1 = faceNormals.get(i*2+j*198);
						fn2 = faceNormals.get(i*2+j*198+1);
						fn3 = faceNormals.get((i-1)*2+j*198);

						x = (fn1.get(0)+fn2.get(0)+fn3.get(0))/3;
						y = (fn1.get(1)+fn2.get(1)+fn3.get(1))/3;
						z = (fn1.get(2)+fn2.get(2)+fn3.get(2))/3;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//upper border without corner vertices
					if(j==terrain[0].length-1 && i>0 && i<terrain.length-1){
						fn1 = faceNormals.get((i-1)*2+(j-1)*198);
						fn2 = faceNormals.get((i-1)*2+(j-1)*198+1);
						fn3 = faceNormals.get(i*2+(j-1)*198+1);

						x = (fn1.get(0)+fn2.get(0)+fn3.get(0))/3;
						y = (fn1.get(1)+fn2.get(1)+fn3.get(1))/3;
						z = (fn1.get(2)+fn2.get(2)+fn3.get(2))/3;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;

						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];
					}
					//all other vertices
					if(j>0 && i>0 && j<terrain[0].length-1 && i<terrain.length-1){
						fn1 = faceNormals.get(i*2+j*198);
						fn2 = faceNormals.get(i*2+j*198+1);
						fn3 = faceNormals.get((i-1)*2+(j-1)*198);
						fn4 = faceNormals.get((i-1)*2+(j-1)*198+1);
						fn5 = faceNormals.get((i-1)*2+j*198);
						fn6 = faceNormals.get(i*2+(j-1)*198+1);

						x = (fn1.get(0)+fn2.get(0)+fn3.get(0)+fn4.get(0)+fn5.get(0)+fn6.get(0))/6;
						y = (fn1.get(1)+fn2.get(1)+fn3.get(1)+fn4.get(1)+fn5.get(1)+fn6.get(1))/6;
						z = (fn1.get(2)+fn2.get(2)+fn3.get(2)+fn4.get(2)+fn5.get(2)+fn6.get(2))/6;

						vertexNormal[0] = x;
						vertexNormal[1] = y;
						vertexNormal[2] = z;


						vNormals[i+j*100][0]= vertexNormal[0];
						vNormals[i+j*100][1]= vertexNormal[1];
						vNormals[i+j*100][2]= vertexNormal[2];

					}
				}
			}

		}


	}
	
	
	/*calculate cross-product of two float-vectors*/
	private FloatBuffer crossProduct(float[]a, float[]b){

		float[] result = new float[3];

		result[0] = a[1]*b[2] - a[2]*b[1];
		result[1] = a[2]*b[0] - a[0]*b[2];
		result[2] = a[0]*b[1] - a[1]*b[0];

		return FloatBuffer.wrap(result);
	}

	public static void main(String args[]) {
		new TerrainColor();
	}




}